/* 
 * Copyright (c) 2010-2011 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: vsim.h,v 1.2 2010/03/05 18:09:27 kamkin Exp $
 */

#include "sync.h"

#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <ucontext.h>
#elif defined(WIN32)
  #include <win32_ucontext.h>
#else
  #error "Unknown platform"
#endif

#ifdef SIGSTACKSIZE
#undef SIGSTACKSIZE
#endif
#define SIGSTACKSIZE (8192*32)

static char stack[SIGSTACKSIZE];

static ucontext_t scenario_context;
static ucontext_t testbench_context;

static int last_operation = 0;

int start_scenario_with_wrapper(void(*wrapper)(void))
{
    getcontext(&scenario_context);
    scenario_context.uc_link = &testbench_context;
    scenario_context.uc_stack.ss_sp = stack;
    scenario_context.uc_stack.ss_size = SIGSTACKSIZE;
    makecontext(&scenario_context, wrapper, 0);
    swapcontext(&testbench_context, &scenario_context);

    return 0;
}

int end_scenario(void)
{
    return 0;
}

int wait_for_apply(void)
{
    return last_operation;
}

// scenario
void wait_for_check(void)
{
    swapcontext(&scenario_context, &testbench_context);
}

// testbench
void apply_check(void)
{
    swapcontext(&testbench_context, &scenario_context);
}

void apply_finish(void)
{
    last_operation = ACTION_FINISH_OPERATION;
}

void apply_operation(void)
{
    last_operation = ACTION_START_OPERATION;
}
