/* 
 * Copyright (c) 2006-2010 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: vsim.h,v 1.2 2010/03/05 18:09:27 kamkin Exp $
 */

#ifndef _VSIM_H_INCLUDED_
#define _VSIM_H_INCLUDED_

#include <vpi_user.h>

#ifdef __cplusplus
extern "C" {
#endif /* #ifdef __cplusplus */

#if !defined (_MSC_VER)
#include <stdint.h>
#endif /* #if !defined (_MSC_VER) */

#if defined (_MSC_VER)

typedef __int8            int8_t;
typedef __int16           int16_t;
typedef __int32           int32_t;
typedef __int64           int64_t;
typedef unsigned __int8   uint8_t;
typedef unsigned __int16  uint16_t;
typedef unsigned __int32  uint32_t;
typedef unsigned __int64  uint64_t;

#endif /* #if defined (_MSC_VER) */

typedef struct int128_t
{
    int64_t h;
    int64_t l;
} int128_t;

typedef struct uint128_t
{
    uint64_t h;
    uint64_t l;
} uint128_t;

typedef struct int256_t
{
    int128_t h;
    int128_t l;
} int256_t;

typedef struct uint256_t
{
    uint128_t h;
    uint128_t l;
} uint256_t;

typedef struct int512_t
{
    int256_t h;
    int256_t l;
} int512_t;

typedef struct uint512_t
{
    uint256_t h;
    uint256_t l;
} uint512_t;

void vsim_putInt512Value(const char *reg, int512_t val);
int512_t vsim_getInt512Value(const char *reg);

void vsim_putUInt512Value(const char *reg, uint512_t val);
uint512_t vsim_getUInt512Value(const char *reg);

void vsim_putInt256Value(const char *reg, int256_t val);
int256_t vsim_getInt256Value(const char *reg);

void vsim_putUInt256Value(const char *reg, uint256_t val);
uint256_t vsim_getUInt256Value(const char *reg);

void vsim_putInt128Value(const char *reg, int128_t val);
int128_t vsim_getInt128Value(const char *reg);

void vsim_putUInt128Value(const char *reg, uint128_t val);
uint128_t vsim_getUInt128Value(const char *reg);

void vsim_putInt64Value(const char *reg, int64_t val);
int64_t vsim_getInt64Value(const char *reg);

void vsim_putUInt64Value(const char *reg, uint64_t val);
uint64_t vsim_getUInt64Value(const char *reg);

void vsim_putInt32Value(const char *reg, int32_t val);
int32_t vsim_getInt32Value(const char *reg);

void vsim_putUInt32Value(const char *reg, uint32_t val);
uint32_t vsim_getUInt32Value(const char *reg);

void vsim_putInt16Value(const char *reg, int16_t val);
int16_t vsim_getInt16Value(const char *reg);

void vsim_putUInt16Value(const char *reg, uint16_t val);
uint16_t vsim_getUInt16Value(const char *reg);

void vsim_putInt8Value(const char *reg, int8_t val);
int8_t vsim_getInt8Value(const char *reg);

void vsim_putUInt8Value(const char *reg, uint8_t val);
uint8_t vsim_getUInt8Value(const char *reg);

#ifdef __cplusplus
}
#endif /* #ifdef __cplusplus */

#endif /* #ifndef _VSIM_H_INCLUDED_ */
