/* 
 * Copyright (c) 2010-2011 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: design.h,v 1.3 2010/03/11 11:54:56 kamkin Exp $
 */

#ifndef _DESIGN_H_INCLUDED_
#define _DESIGN_H_INCLUDED_

typedef enum vt_direct_t
{
    VT_NONE   = 0x000,
    VT_INPUT  = 0x001,
    VT_OUTPUT = 0x002,
    VT_INOUT  = VT_INPUT | VT_OUTPUT,
} vt_direct_t;

typedef enum vt_input_t
{
    VT_DATA  = 0x0000,
    VT_TEST  = 0x0001,
    VT_CLOCK = 0x0002,
    VT_RESET = 0x0003,
} vt_input_t;

typedef struct vt_wire_t
{
    const char *name;
    vt_direct_t direct;
    vt_input_t input;
    unsigned min;
    unsigned max;
    unsigned grouping_counter;
} vt_wire_t;

typedef struct vt_item_t
{
    vt_wire_t *wire;
    struct vt_item_t *next;
} vt_item_t;

typedef struct vt_list_t
{
    size_t size;
    vt_item_t *head;
    vt_item_t *last;
} vt_list_t;

typedef struct vt_design_t
{
    const char *name;
    vt_list_t *inputs;
    vt_list_t *outputs;
} vt_design_t;

void vt_init_wire(vt_wire_t *wire, const char *name, vt_direct_t direct, vt_input_t input, int min, int max);
vt_wire_t* vt_create_wire(const char *name, vt_direct_t direct, vt_input_t input, int min, int max);
void vt_destroy_wire(vt_wire_t *wire);

void vt_init_item(vt_item_t *item, vt_wire_t *wire);
vt_item_t *vt_create_item(vt_wire_t *wire);
void vt_destroy_item(vt_item_t *item);

void vt_init_list(vt_list_t *list);
vt_list_t *vt_create_list(void);
void vt_add_list(vt_list_t *list, vt_wire_t *wire);
void vt_destroy_list(vt_list_t *list);

void vt_init_design(vt_design_t *design, const char *name);
vt_design_t* vt_create_design(const char *name);
void vt_destroy_design(vt_design_t *design);

void vt_add_input_design(vt_design_t *design, vt_wire_t *wire);
void vt_add_output_design(vt_design_t *design, vt_wire_t *wire);

int vt_find_and_update_similar_input_wire_in_design(vt_design_t *design, vt_wire_t *wire);
int vt_find_and_update_similar_output_wire_in_design(vt_design_t *design, vt_wire_t *wire);
void vt_reduce_grouped_wire_names(vt_design_t *design);

#endif // _DESIGN_H_INCLUDED_

