/* 
 * Copyright (c) 2010 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: options.h,v 1.2 2010/03/04 15:40:44 kamkin Exp $
 */

#ifndef _OPTIONS_H_INCLUDED_
#define _OPTIONS_H_INCLUDED_

#include <ivl_target.h>

#define APPLICATION_OPTION  "veritool"
#define CLOCK_INPUT_OPTION  "clk"
#define RESET_INPUT_OPTION  "rst"
#define RST_POSWORK_OPTION  "rstpos"
#define CMODE_OPTION        "c"
#define MODULE_NAME_OPTION  "module"
#define ALL_OUTPUTS_OPTION  "all"
#define TESTBENCH_V_OPTION  "testbench"
#define VPI_MEDIA_C_OPTION  "vpi-media"
#define VPI_MEDIA_H_OPTION  "vpi-media-header"
#define VPI_SYSTF_C_OPTION  "vpi-systf"
#define VPI_SYSTF_H_OPTION  "vpi-systf-header"
#define INTERFACE_C_OPTION  "interface"
#define INTERFACE_H_OPTION  "interface-header"
#define DESTINATION_OPTION  "destination"
#define GROUP_OPTION        "group"

#define EMPTY_OPTION_VALUE  "<DEFAULT>"

#define APPLICATION_DEFAULT "veritool"
#define CLOCK_INPUT_DEFAULT "clk"
#define RESET_INPUT_DEFAULT "rst"
#define RST_POSWORK_DEFAULT 0
#define CMODE_DEFAULT 0
#define MODULE_NAME_DEFAULT ""
#define ALL_OUTPUTS_DEFAULT ""
#define TESTBENCH_V_DEFAULT "testbench.v"
#define VPI_MEDIA_C_DEFAULT "vpi_media.c"
#define VPI_MEDIA_H_DEFAULT "vpi_media.h"
#define VPI_SYSTF_C_DEFAULT "vpi_systf.c"
#define VPI_SYSTF_H_DEFAULT "vpi_systf.h"
#define INTERFACE_C_DEFAULT "interface.c"
#define INTERFACE_H_DEFAULT "interface.h"
#define VPI_MEDIA_CPP_DEFAULT "vpi_media.cpp"
#define VPI_SYSTF_CPP_DEFAULT "vpi_systf.cpp"
#define INTERFACE_CPP_DEFAULT "interface.cpp"
#define DESTINATION_DEFAULT "."
#define GROUP_DEFAULT 0

typedef enum vt_outputs_t
{
    TESTBENCH_V = 0x0001,
    VPI_MEDIA_C = 0x0002,
    VPI_MEDIA_H = 0x0004,
    VPI_SYSTF_C = 0x0008,
    VPI_SYSTF_H = 0x0010,
    INTERFACE_C = 0x0011,
    INTERFACE_H = 0x0012,
    ALL_OUTPUTS = 0xffff,
} vt_outputs_t;

typedef struct vt_options_t
{
    vt_outputs_t outputs;
    const char  *application;
    const char  *clock_input;
    const char  *reset_input;
          char   rst_poswork;
	  char   cmode;
    const char  *module_name;
    const char  *all_outputs;
    const char  *testbench_v;
    const char  *vpi_media_c;
    const char  *vpi_media_h;
    const char  *vpi_systf_c;
    const char  *vpi_systf_h;
    const char  *interface_c;
    const char  *interface_h;
    const char  *destination;
          char   group;
} vt_options_t;

extern vt_options_t options;

int vt_defined_option(const char *value);
int vt_defined_and_empty_option(const char *value);
int vt_defined_and_not_empty_option(const char *value);

int vt_process_options(ivl_design_t des, vt_options_t *options);

#endif // _OPTIONS_H_INCLUDED_
