/* 
 * Copyright (c) 2010 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: printer.c,v 1.2 2010/03/04 15:40:44 kamkin Exp $
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "error.h"
#include "printer.h"
#include "options.h"

#include "printer/testbench.h"
#include "printer/vpi_media.h"
#include "printer/vpi_systf.h"
#include "printer/interface.h"

static void vt_print_header(FILE *file, vt_options_t *options)
{
    fprintf(file, "/*\n");
    fprintf(file, " * Generated by %s: Copyright (c) 2009-2011 ISPRAS\n", options->application);
    fprintf(file, " *\n");
    fprintf(file, " * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)\n");
    fprintf(file, " * 25 A.Solzhenitsyn st. Moscow 109004 Russia\n");
    fprintf(file, " *\n");
    fprintf(file, " * http://hardware.ispras.ru\n");
    fprintf(file, " */\n\n");
}

static char* vt_get_path(char *result, const char *dir, const char *file)
{
    int i = strlen(dir);
	
    strcpy(result, dir);
    result[i] = '/';
    strcpy(result + i + 1, file);
	
    return result;
}

static void vt_print_design(const char *directory, const char *filename,
    void printer(FILE*, vt_design_t*, vt_options_t *options), vt_design_t *design, vt_options_t *options)
{
    FILE *file;
    char path[256];

    assert(directory != NULL);
    assert(filename != NULL);
    
    if(!(file = fopen(vt_get_path(path, directory, filename), "w")))
        { vt_print_error_and_exit("can not write a file", options); }

    vt_print_header(file, options);
    printer(file, design, options);

    fclose(file);
}

void vt_print_files(vt_design_t *design, vt_options_t *options)
{
    assert(design != NULL);
    assert(options != NULL);

    if(options->outputs & TESTBENCH_V) { vt_print_design(options->destination, options->testbench_v, vt_print_testbench_v, design, options); }
    if(options->outputs & VPI_MEDIA_C) { vt_print_design(options->destination, options->vpi_media_c, vt_print_vpi_media_c, design, options); }
    if(options->outputs & VPI_MEDIA_H) { vt_print_design(options->destination, options->vpi_media_h, vt_print_vpi_media_h, design, options); }
    if(options->outputs & VPI_SYSTF_C) { vt_print_design(options->destination, options->vpi_systf_c, vt_print_vpi_systf_c, design, options); }
    if(options->outputs & VPI_SYSTF_H) { vt_print_design(options->destination, options->vpi_systf_h, vt_print_vpi_systf_h, design, options); }
    if(options->outputs & INTERFACE_C) { vt_print_design(options->destination, options->interface_c, vt_print_interface_c, design, options); }
    if(options->outputs & INTERFACE_H) { vt_print_design(options->destination, options->interface_h, vt_print_interface_h, design, options); }
}
