/* 
 * Copyright (c) 2010 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: common.c,v 1.3 2010/03/11 11:21:22 kamkin Exp $
 */

#include <assert.h>
#include <stdlib.h>

#include "common.h"

void vt_print_decorated_comment(FILE *file, const char *comment, vt_options_t *options)
{
    assert(file != NULL);
	
    fprintf(file, "////////////////////////////////////////////////////////////////////////////////\n");
    fprintf(file, "// %s\n", comment);
    fprintf(file, "////////////////////////////////////////////////////////////////////////////////\n");
}

void vt_print_newline(FILE *file)
{
    fprintf(file, "\n");
}

void vt_print_indent(FILE *file, int indent)
{
    while(indent--) { fprintf(file, "    "); }
}

const char* vt_get_suitable_ctype(size_t width)
{
    if(width <= 8)
        { return "uint8_t"; }
    else if(width <= 32)
        { return "uint32_t"; }
    else if(width <= 64)
        { return "uint64_t"; }
    else if(width <= 128)
        { return "uint128_t"; }
    else if(width <= 256)
        { return "uint256_t"; }
    else if(width <= 512)
        { return "uint512_t"; }

    return NULL;    
}

const char* vt_get_suitable_vtype(size_t width)
{
    if(width <= 8)
        { return "UInt8"; }
    else if(width <= 32)
        { return "UInt32"; }
    else if(width <= 64)
        { return "UInt64"; }
    else if(width <= 128)
        { return "UInt128"; }
    else if(width <= 256)
        { return "UInt256"; }
    else if(width <= 512)
        { return "UInt512"; }

    return NULL;    
}

uint64_t vt_get_mask(size_t width)
{
    uint64_t _1 = 1;

    assert(width <= 64);

    return width == 64 ? 0xffffffffffffffffUL : (_1 << width) - 1;
}

unsigned vt_get_lo(uint64_t data)
{
    return (unsigned)(data & 0xffffffffUL);
}

unsigned vt_get_hi(uint64_t data)
{
    return (unsigned)(data >> 32);
}
