/* 
 * Copyright (c) 2010-2011 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: interface.c,v 1.6 2010/06/15 14:53:20 chupilko Exp $
 */

#include <assert.h>
#include <stdio.h>

#include "common.h"
#include "interface.h"

#include "../error.h"

static void vt_print_typedef_interface_h(FILE *file, vt_list_t *list, vt_options_t *options)
{
    vt_item_t *item = list->head;
  	
    while(item)
    {
        vt_wire_t *wire = item->wire;
        
        if(wire->input == VT_DATA)
        {
            //char *name = (char*)wire->name;
            size_t width = wire->max - wire->min + 1;

            if(wire->grouping_counter == 0)
            {
              fprintf(file, "    %s %s; /* %s[%d:%d] */\n", vt_get_suitable_ctype(width),
                  wire->name, wire->name, wire->max, wire->min);
            }
            else
            {
              fprintf(file, "    %s %s[%d]; /* %s_0|%d[%d:%d] */\n", vt_get_suitable_ctype(width),
                  wire->name, wire->grouping_counter + 1, wire->name, wire->grouping_counter, wire->max, wire->min);
            }
        }

        item = item->next;
    }
}

static void vt_print_assign_interface_c(FILE *file, vt_list_t *list, vt_options_t *options)
{
    int i;
    vt_item_t *item = list->head;
	
    while(item)
    {
        vt_wire_t *wire = item->wire;

        if(wire->input == VT_DATA)
        {
            if(wire->grouping_counter == 0)
            {
                if(wire->direct == VT_INPUT)
                    { fprintf(file, "    set_%s_VPIMediator(inputs->%s);\n", wire->name, wire->name); }
                else
                    { fprintf(file, "    outputs->%s = get_%s_VPIMediator();\n", wire->name, wire->name); }
            }
            else
            {
                for(i = 0; i <= wire->grouping_counter; i++)
                {
                    if(wire->direct == VT_INPUT)
                        { fprintf(file, "    set_%s%d_VPIMediator(inputs->%s[%d]);\n", wire->name, i, wire->name, i); }
                    else
                        { fprintf(file, "    outputs->%s[%d] = get_%s%d_VPIMediator();\n", wire->name, i, wire->name, i); }
                }
            }
        }

        item = item->next;
    }
}

void vt_print_interface_h(FILE *file, vt_design_t *design, vt_options_t *options)
{
    assert(file != NULL);
    assert(design != NULL);
    assert(options != NULL);

    fprintf(file, "#ifndef _INTERFACE_H_INCLUDED_\n");
    fprintf(file, "#define _INTERFACE_H_INCLUDED_\n\n");

    fprintf(file, "#include <vsim.h>\n\n");

    fprintf(file, "typedef struct inputs_t\n{\n");
    vt_print_typedef_interface_h(file, design->inputs, options);
    fprintf(file, "} inputs_t;\n\n");

    fprintf(file, "typedef struct outputs_t\n{\n");
    vt_print_typedef_interface_h(file, design->outputs, options);
    fprintf(file, "} outputs_t;\n\n");

    fprintf(file, "void clear_inputs(inputs_t *inputs);\n");
    fprintf(file, "void clear_outputs(outputs_t *outputs);\n\n");

    fprintf(file, "void map_state_down(inputs_t *inputs);\n");
    fprintf(file, "void map_state_up(outputs_t *outputs);\n\n");

    fprintf(file, "#endif // _INTERFACE_H_INCLUDED_\n\n");
}

void vt_print_interface_c(FILE *file, vt_design_t *design, vt_options_t *options)
{
    assert(file != NULL);
    assert(design != NULL);
    assert(options != NULL);

    fprintf(file, "#include <string.h>\n\n");

    fprintf(file, "#include \"%s\"\n",   options->interface_h);
    fprintf(file, "#include \"%s\"\n\n", options->vpi_media_h);

    fprintf(file, "void clear_inputs(inputs_t *inputs)\n{\n");
    fprintf(file, "    memset(inputs, 0, sizeof(inputs_t));\n");
    fprintf(file, "}\n\n");

    fprintf(file, "void clear_outputs(outputs_t *outputs)\n{\n");
    fprintf(file, "    memset(outputs, 0, sizeof(outputs_t));\n");
    fprintf(file, "}\n\n");

    fprintf(file, "void map_state_down(inputs_t *inputs)\n{\n");
    vt_print_assign_interface_c(file, design->inputs, options);
    fprintf(file, "    clear_inputs(inputs);\n");
    fprintf(file, "}\n\n");

    fprintf(file, "void map_state_up(outputs_t *outputs)\n{\n");
    vt_print_assign_interface_c(file, design->outputs, options);
    fprintf(file, "}\n\n");
}
