/* 
 * Copyright (c) 2010-2011 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: testbench.c,v 1.3 2010/03/11 11:54:58 kamkin Exp $
 */

#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "common.h"
#include "testbench.h"

#include "../error.h"

static void vt_print_input_testbench_v(FILE *file, vt_wire_t *wire, vt_options_t *options)
{
    size_t width;

    assert(wire != NULL);

    width = wire->max - wire->min + 1;

    if(wire->input != VT_DATA)
    {
        if(width > 1)
            { fprintf(file, "    reg [%d:%d] %s;\n", wire->max, wire->min, wire->name); }
		else
            { fprintf(file, "    reg %s;\n", wire->name); }
	}
    else
    {
        if(width > 1)
        {
            fprintf(file, "    reg [%d:%d] %s;\n", wire->max, wire->min, wire->name);
            fprintf(file, "    reg [%d:%d] %s_reg;\n", wire->max, wire->min, wire->name);
        }
        else
        {
            fprintf(file, "    reg %s;\n", wire->name);
            fprintf(file, "    reg %s_reg;\n", wire->name);
        }
    }
}

static void vt_print_output_testbench_v(FILE *file, vt_wire_t *wire, vt_options_t *options)
{
    size_t width;

    assert(wire != NULL);

    width = wire->max - wire->min + 1;

    if(width > 1)
        { fprintf(file, "    wire [%d:%d] %s;\n", wire->max, wire->min, wire->name); }
    else
        { fprintf(file, "    wire %s;\n", wire->name); }
}

static void vt_print_reset_testbench_v(FILE *file, vt_wire_t *wire, vt_options_t *options)
{
    int width, i;

    assert(wire != NULL);

    width = wire->max - wire->min + 1;

    fprintf(file, "        %s = %d'h", wire->name, width);

    for(i = 0; i < width >> 2; i++)
        { fprintf(file, "0"); }

    if((width & 0x3) != 0)
    	{ fprintf(file, "0"); }

    fprintf(file, ";\n");
}

static void vt_print_clock_cycle(FILE *file, vt_options_t *options)
{
    fprintf(file, "%s = 0; #(HALF_CYCLE_DELAY); %s = 1; #(HALF_CYCLE_DELAY);\n", options->clock_input, options->clock_input);
}

void vt_print_testbench_v(FILE *file, vt_design_t *design, vt_options_t *options)
{
    vt_item_t *item;

    assert(file != NULL);
    assert(design != NULL);
    assert(options != NULL);

    fprintf(file, "`timescale 1 ns/1 ps\n\n");
    
    fprintf(file, "module testbench();\n\n");
    fprintf(file, "    parameter HALF_CYCLE_DELAY = 5;\n");
    fprintf(file, "    parameter RESET_CYCLE_NUMBER = 3;\n");
    fprintf(file, "    parameter INC_CYCLE_ACCORD_TO_FRONT_OF_CLK = 1;\n\n");

    for(item = design->inputs->head; item; item = item->next)
//        { vt_print_input_testbench_v(file, item->wire, options); }
    {
        if(item->wire->grouping_counter == 0)
          { vt_print_input_testbench_v(file, item->wire, options); }
        else
        {
            int i, len;
            char *tmp;
            for(i = 0; i <= item->wire->grouping_counter; i++)
            {
                tmp = (char*) item->wire->name;
                len = strlen(tmp);
                tmp[len == 0 ? 0 : len] = 0x30 + i;
                tmp[len + 1] = '\0';
                vt_print_input_testbench_v(file, item->wire, options);
                tmp[len == 0 ? 0: len] = '\0';
            }
        }
    }

    fprintf(file, "    reg run;\n");

    for(item = design->outputs->head; item; item = item->next)
//        { vt_print_output_testbench_v(file, item->wire, options); }
    {
        if(item->wire->grouping_counter == 0)
          { vt_print_output_testbench_v(file, item->wire, options); }
        else
        {
            int i, len;
            char *tmp;
            for(i = 0; i <= item->wire->grouping_counter; i++)
            {
                tmp = (char*) item->wire->name;
                len = strlen(tmp);
                tmp[len == 0 ? 0 : len] = 0x30 + i;
                tmp[len + 1] = '\0';
                vt_print_output_testbench_v(file, item->wire, options);
                tmp[len == 0 ? 0: len] = '\0';
            }
        }
    }

    vt_print_newline(file);

    fprintf(file, "    always @(posedge %s) begin\n", options->clock_input);

    for(item = design->inputs->head; item; item = item->next)
    {
        vt_wire_t *wire = item->wire;

        if(wire->input == VT_DATA)
//            { fprintf(file, "        %s_reg <= %s;\n", wire->name, wire->name); }
        {
            if(wire->grouping_counter == 0)
              { fprintf(file, "        %s_reg <= %s;\n", wire->name, wire->name); }
            else
            {
                int i, len;
                char *tmp;
                for(i = 0; i <= wire->grouping_counter; i++)
                {
                    tmp = (char*) wire->name;
                    len = strlen(tmp);
                    tmp[len == 0 ? 0 : len] = 0x30 + i;
                    tmp[len + 1] = '\0';
                    fprintf(file, "        %s_reg <= %s;\n", wire->name, wire->name);
                    tmp[len == 0 ? 0: len] = '\0';
                }
            }
        }


    }

    fprintf(file, "    end\n\n");

    fprintf(file, "    %s target (\n", design->name);

    for(item = design->inputs->head; item; item = item->next)
    {
        vt_wire_t *wire = item->wire;

        if(wire->input == VT_DATA)
//            { fprintf(file, "        .%s(%s_reg)", wire->name, wire->name); }
        {
            if(wire->grouping_counter == 0)
              { fprintf(file, "        .%s(%s_reg)", wire->name, wire->name); }
            else
            {
                int i, len;
                char *tmp;
                for(i = 0; i <= wire->grouping_counter; i++)
                {
                    tmp = (char*) wire->name;
                    len = strlen(tmp);
                    tmp[len == 0 ? 0 : len] = 0x30 + i;
                    tmp[len + 1] = '\0';
                    fprintf(file, "        .%s(%s_reg)", wire->name, wire->name);
                    tmp[len == 0 ? 0: len] = '\0';
                    if(i != wire->grouping_counter)
                        fprintf(file, ",\n");
                }
            }
        }

        else
//            { fprintf(file, "        .%s(%s)", wire->name, wire->name); }
        {
            if(wire->grouping_counter == 0)
              { fprintf(file, "        .%s(%s)", wire->name, wire->name); }
            else
            {
                int i, len;
                char *tmp;
                for(i = 0; i <= wire->grouping_counter; i++)
                {
                    tmp = (char*) wire->name;
                    len = strlen(tmp);
                    tmp[len == 0 ? 0 : len] = 0x30 + i;
                    tmp[len + 1] = '\0';
                    fprintf(file, "        .%s(%s)", wire->name, wire->name);
                    tmp[len == 0 ? 0: len] = '\0';
                    if(i != wire->grouping_counter)
                        fprintf(file, ",\n");
                }
            }
        }

        if(item->next || design->outputs->head)
            { fprintf(file, ",\n"); }
    }

    for(item = design->outputs->head; item; item = item->next)
    {
        vt_wire_t *wire = item->wire;

//        fprintf(file, "        .%s(%s)", wire->name, wire->name);
        if(wire->grouping_counter == 0)
          { fprintf(file, "        .%s(%s)", wire->name, wire->name); }
        else
        {
            int i, len;
            char *tmp;
            for(i = 0; i <= wire->grouping_counter; i++)
            {
                tmp = (char*) wire->name;
                len = strlen(tmp);
                tmp[len == 0 ? 0 : len] = 0x30 + i;
                tmp[len + 1] = '\0';
                fprintf(file, "        .%s(%s)", wire->name, wire->name);
                tmp[len == 0 ? 0: len] = '\0';
                if(i != wire->grouping_counter)
                    fprintf(file, ",\n");
            }
        }

        if(item->next)
            { fprintf(file, ",\n"); }
    }

    fprintf(file, "\n    );\n\n");

    fprintf(file, "    reg [63 : 0] CYCLE;\n");
    fprintf(file, "    reg [256 * 8 : 1] argv;\n");
    fprintf(file, "    reg [64 * 8 : 1] pstring;\n");
    fprintf(file, "    reg [64 * 8 : 1] dumpfilename;\n");
    fprintf(file, "    reg netscen;\n\n");

    fprintf(file, "    task cycle;\n");
    fprintf(file, "    input inc_CYCLE;\n");
    fprintf(file, "    begin\n");

    fprintf(file, "        %s = 0; #(HALF_CYCLE_DELAY);\n", options->clock_input);
    fprintf(file, "        if(inc_CYCLE == 1 && INC_CYCLE_ACCORD_TO_FRONT_OF_CLK == 1) CYCLE = CYCLE + 1;\n");
    fprintf(file, "        %s = 1; #(HALF_CYCLE_DELAY);\n", options->clock_input);
    fprintf(file, "        if(inc_CYCLE == 1 && INC_CYCLE_ACCORD_TO_FRONT_OF_CLK == 0) CYCLE = CYCLE + 1;\n");

    fprintf(file, "    end\n");
    fprintf(file, "    endtask\n\n");

    fprintf(file, "    task delay;\n");
    fprintf(file, "    input inc_CYCLE;\n");
    fprintf(file, "    input [31:0] number_of_cycles;\n");
    fprintf(file, "    reg [31:0] i;\n");
    fprintf(file, "    begin\n");
    fprintf(file, "        i = 0;\n");
    fprintf(file, "        while(i < number_of_cycles)\n");
    fprintf(file, "        begin\n");
    fprintf(file, "            cycle(inc_CYCLE);\n");
    fprintf(file, "            i = i + 1;\n");
    fprintf(file, "        end\n");
    fprintf(file, "    end\n");
    fprintf(file, "    endtask\n\n");

    fprintf(file, "    initial begin\n");
    fprintf(file, "        if(!$value$plusargs(\"dumpfile=%ss\", dumpfilename)) begin\n", "%");
    fprintf(file, "            dumpfilename=\"testbench.vcd\";\n");
    fprintf(file, "        end\n");
    fprintf(file, "        if(!$test$plusargs(\"nodumpfile\")) begin\n");
    fprintf(file, "            $dumpfile(dumpfilename);\n");
    fprintf(file, "            $dumpvars(1, testbench);\n");
    fprintf(file, "        end\n");

    for(item = design->inputs->head; item; item = item->next)
//        { vt_print_reset_testbench_v(file, item->wire, options); }
    {
        if(item->wire->grouping_counter == 0)
          { vt_print_reset_testbench_v(file, item->wire, options); }
        else
        {
            int i, len;
            char *tmp;
            for(i = 0; i <= item->wire->grouping_counter; i++)
            {
                tmp = (char*) item->wire->name;
                len = strlen(tmp);
                tmp[len == 0 ? 0 : len] = 0x30 + i;
                tmp[len + 1] = '\0';
                vt_print_reset_testbench_v(file, item->wire, options);
                tmp[len == 0 ? 0: len] = '\0';
            }
        }
    }


    fprintf(file, "        CYCLE = 0;\n");
    fprintf(file, "        %s = 1'h%d;\n", options->reset_input, options->rst_poswork);

    fprintf(file, "        delay(0, RESET_CYCLE_NUMBER);\n\n");

    fprintf(file, "        %s = 1'h%d;\n\n", options->reset_input, options->rst_poswork == 0 ? 1 : 0);

    fprintf(file, "        if($test$plusargs(\"netscen\"))\n");
    fprintf(file, "            netscen = 1;\n");
    fprintf(file, "        else\n");
    fprintf(file, "            netscen = 0;\n\n");
    fprintf(file, "        argv = 0;\n");
    fprintf(file, "        if($value$plusargs(\"argv=%ss\", argv)) begin\n", "%");
    fprintf(file, "        end\n\n");
    fprintf(file, "        if($value$plusargs(\"scen=%ss\", pstring)) begin\n", "%");
    fprintf(file, "            $setScenario(netscen, argv, pstring);\n");
    fprintf(file, "            run = 1'h1;\n");
    fprintf(file, "        end\n");
    fprintf(file, "        else begin\n");
    fprintf(file, "            $display(\"Please, type a scenario name with +scen=SCEN_NAME parameter\");\n");
    fprintf(file, "            run = 1'h0;\n");
    fprintf(file, "        end\n\n");

    fprintf(file, "        while(run) begin\n");
    fprintf(file, "            cycle(1);\n");
    fprintf(file, "            $callts();\n");
    fprintf(file, "        end\n");

    fprintf(file, "        cycle(1);\n");

    fprintf(file, "    end\n");
    fprintf(file, "endmodule\n\n");
}
