/* 
 * Copyright (c) 2010-2011 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 * http://hardware.ispras.ru
 *
 * All rights reserved.
 *
 * This source code is free software; you can redistribute it
 * and/or modify it in source code form under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 * $Id: vpi_media.c,v 1.10 2010/06/15 14:53:20 chupilko Exp $
 */

#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "common.h"
#include "vpi_media.h"

#include "../error.h"

#ifdef __GNUC__
#define INT_CONST "UL"
#define LONG_CONST "ULL"
#else
#define INT_CONST "U"
#define LONG_CONST "UL"
#endif

static void vt_print_input_vpi_media_c(FILE *file, vt_wire_t *wire, vt_options_t *options)
{
    size_t width;

    assert(wire != NULL);

    width = wire->max - wire->min + 1;

    assert(width > 0 && width <= 512);

    fprintf(file, "void set_%s_VPIMediator(%s %s)\n{\n", wire->name, vt_get_suitable_ctype(width), wire->name);

    if(width > 256)
    {
        if(width != 512)
	{
	    uint64_t mask;
	    
	    if(width > 458)
	    {
	        mask = vt_get_mask(width - 458);
		if(width > 480)
		    { fprintf(file, "    %s.h.h.h &= 0x%x%x%s;\n", wire->name, vt_get_hi(mask), vt_get_lo(mask), LONG_CONST); }
		else
		    { fprintf(file, "    %s.h.h.h &= 0x%x%s;\n",    wire->name, vt_get_lo(mask), INT_CONST); }
	    }
	    else if(width > 394)
	    {
	        mask = vt_get_mask(width - 394);
		fprintf(file, "    %s.h.h.h &= 0x0%s;\n", wire->name, INT_CONST);
		if(width > 426)
		    { fprintf(file, "    %s.h.h.l &= 0x%x%x%s;\n", wire->name, vt_get_hi(mask), vt_get_lo(mask), LONG_CONST); }
		else
		    { fprintf(file, "    %s.h.h.l &= 0x%x%s;\n",    wire->name, vt_get_lo(mask), INT_CONST); }
	    }
	    else if(width > 330)
	    {
	        mask = vt_get_mask(width - 330);
		fprintf(file, "    %s.h.h.h &= 0x0%s;\n", wire->name, INT_CONST);
		fprintf(file, "    %s.h.h.l &= 0x0%s;\n", wire->name, INT_CONST);
		if(width > 352)
		    { fprintf(file, "    %s.h.l.h &= 0x%x%x%s;\n", wire->name, vt_get_hi(mask), vt_get_lo(mask), LONG_CONST); }
		else
		    { fprintf(file, "    %s.h.l.h &= 0x%x%s;\n",    wire->name, vt_get_lo(mask), INT_CONST); }
	    }
	    else if(width > 256)
	    {
	        mask = vt_get_mask(width - 256);
		fprintf(file, "    %s.h.h.h &= 0x0%s;\n", wire->name, INT_CONST);
		fprintf(file, "    %s.h.h.l &= 0x0%s;\n", wire->name, INT_CONST);
		fprintf(file, "    %s.h.l.h &= 0x0%s;\n", wire->name, INT_CONST);
		if(width > 298)
		    { fprintf(file, "    %s.h.l.l &= 0x%x%x%s;\n", wire->name, vt_get_hi(mask), vt_get_lo(mask), LONG_CONST); }
		else
		    { fprintf(file, "    %s.h.l.l &= 0x%x%s;\n", wire->name, vt_get_lo(mask), INT_CONST); }
	    }
	}
    }
    else if(width > 192)
    {
        if(width != 256)
        {
            uint64_t mask = vt_get_mask(width - 192);
        
            if(width > 224)
                { fprintf(file, "    %s.h.h &= 0x%x%x%s;\n", wire->name, vt_get_hi(mask), vt_get_lo(mask), LONG_CONST); }
            else
                { fprintf(file, "    %s.h.h &= 0x%x%s;\n",    wire->name, vt_get_lo(mask), INT_CONST); }
        }
    }
    else if(width > 128)
    {
        fprintf(file, "    %s.h.h  = 0x0%s;\n", wire->name, INT_CONST);

        if(width != 192)
        {
            uint64_t mask = vt_get_mask(width - 128);
        
            if(width > 160)
                { fprintf(file, "    %s.h.l &= 0x%x%x%s;\n", wire->name, vt_get_hi(mask), vt_get_lo(mask), LONG_CONST); }
            else
                { fprintf(file, "    %s.h.l &= 0x%x%s;\n",   wire->name, vt_get_lo(mask), INT_CONST); }
        }
    }
    else if(width > 64)
    {
        if(width != 128)
        {
            uint64_t mask = vt_get_mask(width - 64);
        
            if(width > 96)
                { fprintf(file, "    %s.h &= 0x%x%x%s;\n", wire->name, vt_get_hi(mask), vt_get_lo(mask), LONG_CONST); }
            else
                { fprintf(file, "    %s.h &= 0x%x%s;\n",   wire->name, vt_get_lo(mask), INT_CONST); }
        }
    }
    else if(width > 32)
    {
        if(width != 64)
        {
            uint64_t mask = vt_get_mask(width);
         
            fprintf(file, "    %s &= 0x%x%x%s;\n", wire->name, vt_get_hi(mask), vt_get_lo(mask), LONG_CONST);
        }
    }
    else if(width > 8)
    {
        if(width != 32)
        {
            uint64_t mask = vt_get_mask(width);
        
            fprintf(file, "    %s &= 0x%x%s;\n", wire->name, vt_get_lo(mask), INT_CONST);
        }
    }
    else if(width > 0)
    {
        if(width != 8)
        {
            uint64_t mask = vt_get_mask(width);
        
            fprintf(file, "    %s &= 0x%x%s;\n", wire->name, vt_get_lo(mask), INT_CONST);
        }
    }

    fprintf(file, "    vsim_put%sValue(TESTBENCH_WIRE_NAME(\"%s\"), %s);\n}\n\n", vt_get_suitable_vtype(width), wire->name, wire->name);
}

static void vt_print_output_vpi_media_c(FILE *file, vt_wire_t *wire, vt_options_t *options)
{
    size_t width;
    
    assert(wire != NULL);

    width = wire->max - wire->min + 1;

    if(width > 512)
        { vt_print_error_and_exit("too large wire", options); }

    fprintf(file, "%s get_%s_VPIMediator(void)\n{\n", vt_get_suitable_ctype(width), wire->name);
    fprintf(file, "    return vsim_get%sValue(TESTBENCH_WIRE_NAME(\"%s\"));\n}\n\n", vt_get_suitable_vtype(width), wire->name);
}

static void vt_print_input_vpi_media_h(FILE *file, vt_wire_t *wire, vt_options_t *options)
{
    size_t width;

    assert(wire != NULL);

    width = wire->max - wire->min + 1;

    fprintf(file, "void set_%s_VPIMediator(%s %s);\n", wire->name, vt_get_suitable_ctype(width), wire->name);
}

static void vt_print_output_vpi_media_h(FILE *file, vt_wire_t *wire, vt_options_t *options)
{
    size_t width;
	
    assert(wire != NULL);

    width = wire->max - wire->min + 1;

    fprintf(file, "%s get_%s_VPIMediator(void);\n", vt_get_suitable_ctype(width), wire->name);
}

void vt_print_vpi_media_c(FILE *file, vt_design_t *design, vt_options_t *options)
{
    vt_item_t *item;
    
    vt_wire_t run = { .name="run", .direct=VT_INPUT, .input=VT_TEST, .min=0, .max=0 };

    assert(file != NULL);
    assert(design != NULL);
    assert(options != NULL);

    fprintf(file, "#include \"%s\"\n\n", options->vpi_media_h);

    if(design->inputs->size)
        { vt_print_decorated_comment(file, "Input Signals", options); }
    
    for(item = design->inputs->head; item; item = item->next)
    {
        vt_wire_t *wire = item->wire;
		
        if(wire->input == VT_DATA || wire->input == VT_TEST)
        {
            if(item->wire->grouping_counter == 0)
            {
                vt_print_input_vpi_media_c(file, item->wire, options);
            }
            else
            {
                int i, len;
                char *tmp;
                for(i = 0; i <= wire->grouping_counter; i++)
                {
                    tmp = (char*) wire->name;
                    len = strlen(tmp);
                    tmp[len == 0 ? 0 : len] = 0x30 + i;
                    tmp[len + 1] = '\0';
                    vt_print_input_vpi_media_c(file, wire, options);
                    tmp[len == 0 ? 0: len] = '\0';
                }
            }
        }
    }

    if(design->outputs->size)
        { vt_print_decorated_comment(file, "Output Signals", options); }
    
    for(item = design->outputs->head; item; item = item->next)
    {
        vt_wire_t *wire = item->wire;
		
        if(wire->input == VT_DATA || wire->input == VT_TEST)
        {
            if(item->wire->grouping_counter == 0)
            {
                vt_print_output_vpi_media_c(file, item->wire, options);
            }
            else
            {
                int i, len;
                char *tmp;
                for(i = 0; i <= wire->grouping_counter; i++)
                {
                    tmp = (char*) wire->name;
                    len = strlen(tmp);
                    tmp[len == 0 ? 0 : len] = 0x30 + i;
                    tmp[len + 1] = '\0';
                    vt_print_output_vpi_media_c(file, wire, options);
                    tmp[len == 0 ? 0: len] = '\0';
                }
            }
        }
    }

    vt_print_decorated_comment(file, "Testbench Signals", options);
    vt_print_input_vpi_media_c(file, &run, options);

    fprintf(file, "void start_testbench(void) { set_%s_VPIMediator(1); }\n", run.name);
    fprintf(file, "void stop_testbench(void) { set_%s_VPIMediator(0); }\n", run.name);
}

void vt_print_vpi_media_h(FILE *file, vt_design_t *design, vt_options_t *options)
{
    vt_item_t *item;
    vt_wire_t run = { .name="run", .direct=VT_INPUT, .input=VT_TEST, .min=0, .max=0 };

    assert(file != NULL);
    assert(design != NULL);
    assert(options != NULL);

    fprintf(file, "#ifndef _VPI_MEDIA_H_INCLUDED_\n");
    fprintf(file, "#define _VPI_MEDIA_H_INCLUDED_\n\n");
	
    fprintf(file, "#include <vsim.h>\n\n");
	
    fprintf(file, "#define TESTBENCH_NAME                 \"testbench\"\n");
    fprintf(file, "#define WIRE_NAME(module, wire)        module \".\" wire\n");
    fprintf(file, "#define TESTBENCH_WIRE_NAME(wire)      WIRE_NAME(TESTBENCH_NAME, wire)\n\n");

    if(design->inputs->size)
        { vt_print_decorated_comment(file, "Input Signals", options); }
    for(item = design->inputs->head; item; item = item->next)
    {
        vt_wire_t *wire = item->wire;

        if(wire->input == VT_DATA || wire->input == VT_TEST)
        {
            if(item->wire->grouping_counter == 0)
            {
                vt_print_input_vpi_media_h(file, item->wire, options);
            }
            else
            {
                int i, len;
                char *tmp;
                for(i = 0; i <= wire->grouping_counter; i++)
                {
                    tmp = (char*) wire->name;
                    len = strlen(tmp);
                    tmp[len == 0 ? 0 : len] = 0x30 + i;
                    tmp[len + 1] = '\0';
                    vt_print_input_vpi_media_h(file, wire, options);
                    tmp[len == 0 ? 0: len] = '\0';
                }
            }
        }
    }

    if(design->inputs->size)
        { vt_print_newline(file); }

    if(design->outputs->size)
        { vt_print_decorated_comment(file, "Output Signals", options); }

    for(item = design->outputs->head; item; item = item->next)
    {
        vt_wire_t *wire = item->wire;

        if(wire->input == VT_DATA || wire->input == VT_TEST)
        {
            if(item->wire->grouping_counter == 0)
            {
                vt_print_output_vpi_media_h(file, item->wire, options);
            }
            else
            {
                int i, len;
                char *tmp;
                for(i = 0; i <= wire->grouping_counter; i++)
                {
                    tmp = (char*) wire->name;
                    len = strlen(tmp);
                    tmp[len == 0 ? 0 : len] = 0x30 + i;
                    tmp[len + 1] = '\0';
                    vt_print_output_vpi_media_h(file, wire, options);
                    tmp[len == 0 ? 0: len] = '\0';
                }
            }
        }
    }

    if(design->outputs->size)
        { vt_print_newline(file); }

    vt_print_decorated_comment(file, "Testbench Signals", options);
    vt_print_input_vpi_media_h(file, &run, options);

    vt_print_newline(file);
    fprintf(file, "void start_testbench(void);\n");
    fprintf(file, "void stop_testbench(void);\n");

    vt_print_newline(file);

    fprintf(file, "#endif // _VPI_MEDIA_H_INCLUDED_\n\n");
}
